<?php

/**
 * @file
 * Defines page and form callbacks.
 */

/**
 * Mass user update form, first step
 * Load a file and extract the labels. Prepare for the second step.
 */
function mass_user_update_update_form() {
  $form = array(
    '#attributes' => array('enctype' => 'multipart/form-data')
  );

  $form['mass_user_update_file'] = array(
    '#type' => 'file',
    '#title' => t("Select file to import"),
    '#default_value' => 0,
    '#description' => t("Allowed extensions: @formats", array('@formats' => implode(', ', mass_user_update_allowed_formats()))),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t("Import file"),
  );

  return $form;
}

/**
 * Step 1 validation
 */
function mass_user_update_update_form_validate($form, &$form_state) {
 if (isset($_FILES['files']) && is_uploaded_file($_FILES['files']['tmp_name']['mass_user_update_file'])) {
    $file = file_save_upload('mass_user_update_file');

    if (!$file) {
      form_set_error('mass_user_update_file', t("Error saving uploaded file."));
      return;
    }
    else {
      if (!in_array(strtolower(array_pop(explode('.', $file->filename))), mass_user_update_allowed_formats(), TRUE)) {
        form_set_error('mass_user_update_file', t("Only files in the following formats are allowed: @formats", array('@formats' => implode(', ', mass_user_update_allowed_formats()))));
      }
      else {
        $form_state['file'] = $file;
      }
    }
  }
  else {
    form_set_error('i18n_importexport_file', t("Error uploading file."));
    return;
  }
}

/**
 * Step 1 submission
 */
function mass_user_update_update_form_submit($form, $form_state) {
  drupal_goto("admin/user/user/mass-update/{$form_state['file']->fid}");
}

/**
 * Mass user update form, second step.
 * Allow admins to mapp columns to user fields.
 */
function mass_user_update_form($form_state, $fid) {
  $file = db_fetch_object(db_query("SELECT * FROM {files} WHERE fid = %d", $fid));

  $form = array();
  $fields = array();
  $properties = array();

  foreach(mass_user_update_fetch('fields', $file) as $field) {
    $fields[$field] = $field;
  }

  foreach (mass_user_update_mappable_properties() as $property) {
    $properties[$property] = $property;
  }

  $form['#fields'] = $fields;
  $form['#properties'] = $properties;
  $form['#file'] = $file;

  $form['user_identification'] = array(
    '#type' => 'fieldset',
    '#title' => t("User identification")
  );

  $form['user_identification']['user_identifier_field'] = array(
    '#type' => 'select',
    '#title' => t("Identify user by"),
    '#description' => t("Select which field in the imported file must be used to find a user in the database."),
    '#options' => $fields,
    '#required' => TRUE,
  );

  $form['user_identification']['user_identifier_property'] = array(
    '#type' => 'select',
    '#title' => t("Field corresponds to"),
    '#description' => t("Select to what the previously selected field must be mapped in order to find the user in the database."),
    '#options' => $properties,
    '#required' => TRUE,
  );

  $form['map_fields'] = array(
    '#type' => 'fieldset',
    '#title' => t("Update users by mapping fields")
  );

  foreach ($fields as $field) {
    $form['map_fields']["field_{$field}"] = array(
      '#type' => 'select',
      '#title' => t("Map <em>@field</em> to", array('@field' => $field)),
      '#description' => t("Map the <em>@field</em> from the imported file to which user property ?", array('@field' => $field)),
      '#options' => array('none' => t("--None--")) + $properties
    );
  }

  $form['test_import'] = array(
    '#type' => 'button',
    '#name' => 'test',
    '#value' => t("Test")
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t("Update users")
  );

  return $form;
}

/**
 * Step 2, validation and test
 */
function mass_user_update_form_validate($form, $form_state) {
  $errors = form_get_errors();

  if (empty($errors) && $form_state['clicked_button']['#name'] == 'test') {
    $map = _mass_user_update_form_field_mapping($form, $form_state);

    $updates = mass_user_update_test_update($form['#file'], $form_state['values']['user_identifier_field'], $form_state['values']['user_identifier_property'], $map);

    // @todo
    // Make into theme() call !!!
    if (!empty($updates)) {
      $user_names = array();      

      $update_list = '<ul>';

      foreach ($updates as $uid => $mods) {
        $user_names[$uid] = t("@name (user @uid)", array(
          '@name' => db_result(db_query("SELECT name FROM {users} WHERE uid = %d", $uid)),
          '@uid' => $uid
        ));
        
        $update_list .= '<li>' . t("For user <em>!user</em>:", array('!user' => $user_names[$uid])) . '<ul>';

        foreach ($mods as $update) {
          $update_list .= '<li>' . check_plain($update['property']) . ': ' . check_plain($update['old_value']) . ' => ' . check_plain($update['new_value']) . '</li>';
        }

        $update_list .= '</ul></li>';
      }

      $update_list .= '</ul>';
      
      $user_list = '<ul><li>' . implode('</li><li>', $user_names) . '</li></ul>';
      
      drupal_set_message(t("Identified the following users: !user_list", array('!user_list' => $user_list)));

      drupal_set_message(t("Would proceed to the following updates: !update_list", array('!update_list' => $update_list)));
    }
    else {
      drupal_set_message(t("Could not identify any users !"), 'warning');
    }
  }
}

/**
 * Helper function to extract the selected mapping and assign it to an array.
 *
 * @param array $form
 * @param array $form_state
 * @return array
 */
function _mass_user_update_form_field_mapping($form, $form_state) {
  $map = array();
  
  foreach ($form['#fields'] as $field) {
    if ($form_state['values']["field_{$field}"] != 'none') {
      $map[$field] = $form_state['values']["field_{$field}"];
    }
  }
  
  return $map;
}

/**
 * Step 2, submission and batch start
 */
function mass_user_update_form_submit($form, $form_state) {
  $map = _mass_user_update_form_field_mapping($form, $form_state);

  mass_user_update_launch_update($form['#file'], $form_state['values']['user_identifier_field'], $form_state['values']['user_identifier_property'], $map);
}
